/* ******************************************************************* *\
*                                dgwrap.d                               *
*                                                                       *
*                       Stewart's Utility Library                       *
*                                                                       *
*                              Version 0.10                             *
*                                                                       *
*                 Delegate wrapper for function pointer                 *
>***********************************************************************<
* Copyright (C) 2009-2012 Stewart Gordon.                               *
*                                                                       *
* This software is provided 'as-is', without any express or implied     *
* warranty.  In no event will the author be held liable for any damages *
* arising from the use of this software.                                *
*                                                                       *
* Permission is granted to anyone to use this software for any purpose, *
* including commercial applications, and to alter it and redistribute   *
* it freely, subject to the following restrictions:                     *
*                                                                       *
* - The origin of this software must not be misrepresented; you must    *
*   not claim that you wrote the original software.  If you use this    *
*   software in a product, an acknowledgment in the product             *
*   documentation would be appreciated but is not required.             *
* - Altered source versions must be plainly marked as such, and must    *
*   not be misrepresented as being the original software.               *
* - This notice may not be removed or altered from any source           *
*   distribution.                                                       *
*                                                                       *
* For updates, documentation and other information related to this      *
* product, visit http://pr.stewartsplace.org.uk/d/sutil/.               *
* Email me with comments/suggestions/bug reports: smjg@iname.com        *
\* ******************************************************************* */
module smjg.libs.util.dgwrap;

/*!	\file     dgwrap.d
 *	\brief    Delegate wrapper for function pointer
 */

/*!	\package  smjg.libs.util.dgwrap
 *	\brief    Delegate wrapper for function pointer
 *
 *	The smjg.libs.util.dgwrap module provides the DelegateWrapper
 *	template, used to create a delegate from a function pointer.
 *	This makes it possible to use pointers to global or static member
 *	functions in circumstances where a delegate is required.
 *
 *	\limit    Doesn't work on variadic functions or functions with non-D
 *	          linkage.
 *
 *	\since    0.04
 */


private struct DW(Return, Params...) {
	Return function(Params) fn;

	Return call(Params p) {
		return fn(p);
	}
}


//!	The DelegateWrapper template function creates a delegate wrapper for
//!	the given function pointer.
template DelegateWrapper(Return, Params...) {
	Return delegate(Params) DelegateWrapper(Return function(Params) fn) {
		DW!(Return, Params)* dw = new DW!(Return, Params);
		dw.fn = fn;
		return &dw.call;
	}
}


unittest {
	static string pangram() {
		return "Jackdaws love my big sphinx of quartz.";
	}

	static int count(string s) {
		return s.length;
	}

	static int mean(int x, int y) {
		return (x + y) / 2;
	}

	string delegate() dgPangram = DelegateWrapper(&pangram);
	int delegate(string) dgCount = DelegateWrapper(&count);
	int delegate(int, int) dgMean = DelegateWrapper(&mean);

	assert (dgCount(dgPangram()) == 38);
	assert (dgMean(42, 234) == 138);
}
